@@ -180,6 +180,7 @@ pub(crate) struct ArgumentAnalysis {
     pub(crate) was_reference: bool,
     pub(crate) deps: HashSet<QualifiedName>,
     pub(crate) requires_unsafe: UnsafetyNeeded,
+    pub(crate) is_placement_return_destination: bool,
 }
 
 struct ReturnTypeAnalysis {
@@ -326,6 +327,7 @@ impl<'a> FnAnalyzer<'a> {
                     ..
                 } => Some(api.name().clone()),
                 Api::Enum { .. } => Some(api.name().clone()),
+                Api::ExternCppType { pod: true, .. } => Some(api.name().clone()),
                 _ => None,
             })
             .chain(
@@ -722,6 +724,7 @@ impl<'a> FnAnalyzer<'a> {
                     true,
                     None,
                     sophistication,
+                    false,
                 )
             })
             .partition(Result::is_ok);
@@ -999,6 +1002,7 @@ impl<'a> FnAnalyzer<'a> {
                     &mut param_details,
                     None,
                     sophistication,
+                    true,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1016,6 +1020,7 @@ impl<'a> FnAnalyzer<'a> {
                     &mut param_details,
                     Some(RustConversionType::FromTypeToPtr),
                     sophistication,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1038,6 +1043,7 @@ impl<'a> FnAnalyzer<'a> {
                     &mut param_details,
                     Some(RustConversionType::FromPinMaybeUninitToPtr),
                     sophistication,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1061,6 +1067,7 @@ impl<'a> FnAnalyzer<'a> {
                     &mut param_details,
                     Some(RustConversionType::FromPinMaybeUninitToPtr),
                     sophistication,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
                 self.reanalyze_parameter(
@@ -1072,6 +1079,7 @@ impl<'a> FnAnalyzer<'a> {
                     &mut param_details,
                     Some(RustConversionType::FromPinMoveRefToPtr),
                     sophistication,
+                    false,
                 )
                 .unwrap_or_else(&mut set_ignore_reason);
             }
@@ -1410,6 +1418,7 @@ impl<'a> FnAnalyzer<'a> {
         param_details: &mut [ArgumentAnalysis],
         force_rust_conversion: Option<RustConversionType>,
         sophistication: TypeConversionSophistication,
+        construct_into_self: bool,
     ) -> Result<(), ConvertError> {
         self.convert_fn_arg(
             fun.inputs.iter().nth(param_idx).unwrap(),
@@ -1420,6 +1429,7 @@ impl<'a> FnAnalyzer<'a> {
             false,
             force_rust_conversion,
             sophistication,
+            construct_into_self,
         )
         .map(|(new_arg, new_analysis)| {
             param_details[param_idx] = new_analysis;
@@ -1574,6 +1584,7 @@ impl<'a> FnAnalyzer<'a> {
         treat_this_as_reference: bool,
         force_rust_conversion: Option<RustConversionType>,
         sophistication: TypeConversionSophistication,
+        construct_into_self: bool,
     ) -> Result<(FnArg, ArgumentAnalysis), ConvertError> {
         Ok(match arg {
             FnArg::Typed(pt) => {
@@ -1582,6 +1593,7 @@ impl<'a> FnAnalyzer<'a> {
                 let old_pat = *pt.pat;
                 let mut treat_as_reference = false;
                 let mut treat_as_rvalue_reference = false;
+                let mut is_placement_return_destination = false;
                 let new_pat = match old_pat {
                     syn::Pat::Ident(mut pp) if pp.ident == "this" => {
                         let this_type = match pt.ty.as_ref() {
@@ -1622,6 +1634,7 @@ impl<'a> FnAnalyzer<'a> {
                             ))),
                         }?;
                         self_type = Some(this_type);
+                        is_placement_return_destination = construct_into_self;
                         if treat_this_as_reference {
                             pp.ident = Ident::new("self", pp.ident.span());
                             treat_as_reference = true;
@@ -1673,6 +1686,7 @@ impl<'a> FnAnalyzer<'a> {
                         ),
                         deps: annotated_type.types_encountered,
                         requires_unsafe,
+                        is_placement_return_destination,
                     },
                 )
             }
@@ -1756,20 +1770,6 @@ impl<'a> FnAnalyzer<'a> {
         }
     }
 
-    fn return_type_conversion_details(&self, ty: &Type) -> TypeConversionPolicy {
-        match ty {
-            Type::Path(p) => {
-                let tn = QualifiedName::from_type_path(p);
-                if self.pod_safe_types.contains(&tn) {
-                    TypeConversionPolicy::new_unconverted(ty.clone())
-                } else {
-                    TypeConversionPolicy::new_to_unique_ptr(ty.clone())
-                }
-            }
-            _ => TypeConversionPolicy::new_unconverted(ty.clone()),
-        }
-    }
-
     fn convert_return_type(
         &mut self,
         rt: &ReturnType,
@@ -1789,7 +1789,18 @@ impl<'a> FnAnalyzer<'a> {
                     self.convert_boxed_type(boxed_type.clone(), ns, references.ref_return)?;
                 let boxed_type = annotated_type.ty;
                 let was_reference = matches!(boxed_type.as_ref(), Type::Reference(_));
-                let conversion = self.return_type_conversion_details(boxed_type.as_ref());
+                let ty: &Type = boxed_type.as_ref();
+                let conversion = match ty {
+                    Type::Path(p) => {
+                        let tn = QualifiedName::from_type_path(p);
+                        if self.pod_safe_types.contains(&tn) {
+                            TypeConversionPolicy::new_unconverted(ty.clone())
+                        } else {
+                            TypeConversionPolicy::new_to_unique_ptr(ty.clone())
+                        }
+                    }
+                    _ => TypeConversionPolicy::new_unconverted(ty.clone()),
+                };
                 ReturnTypeAnalysis {
                     rt: ReturnType::Type(*rarrow, boxed_type),
                     conversion: Some(conversion),
