@@ -1,22 +1,18 @@
 // Copyright 2020 Google LLC
 //
-// Licensed under the Apache License, Version 2.0 (the "License");
-// you may not use this file except in compliance with the License.
-// You may obtain a copy of the License at
-//
-//    https://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing, software
-// distributed under the License is distributed on an "AS IS" BASIS,
-// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-// See the License for the specific language governing permissions and
-// limitations under the License.
+// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
+// https://www.apache.org/licenses/LICENSE-2.0> or the MIT license
+// <LICENSE-MIT or https://opensource.org/licenses/MIT>, at your
+// option. This file may not be copied, modified, or distributed
+// except according to those terms.
 
 use autocxx_parser::UnsafePolicy;
 #[allow(unused_imports)]
 use syn::parse_quote;
 use syn::ItemMod;
 
+use crate::CppCodegenOptions;
+
 use super::BridgeConverter;
 
 // This mod is for tests which take bindgen output directly.
@@ -33,8 +29,13 @@ fn do_test(input: ItemMod) {
     let tc = parse_quote! {};
     let bc = BridgeConverter::new(&[], &tc);
     let inclusions = "".into();
-    bc.convert(input, UnsafePolicy::AllFunctionsSafe, inclusions, false)
-        .unwrap();
+    bc.convert(
+        input,
+        UnsafePolicy::AllFunctionsSafe,
+        inclusions,
+        &CppCodegenOptions::default(),
+    )
+    .unwrap();
 }
 
 // How to add a test here
